/*
 * QUARTZ window driver
 *
 * Copyright 1998,1999 Patrik Stridvall
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config.h"

#include "gdi.h"
#include "quartzdrv.h"
#include "win.h"
#include "winpos.h"
#include "wownt32.h"
#include "wine/debug.h"

WINE_DEFAULT_DEBUG_CHANNEL(quartzdrv);

extern void *qd_topmost_group, *qd_nontopmost_group;
void* QUARTZDRV_CreateNewWindow(const char *const title, int x, int y,
								int width, int height);

/**********************************************************************
 *		CreateWindow   (QUARTZDRV.@)
 */
BOOL QUARTZDRV_CreateWindow( HWND hwnd, CREATESTRUCTA *cs, BOOL unicode )
{
  BOOL ret;
  HWND hwndLinkAfter;
  CBT_CREATEWNDA cbtc;
  WND *wndPtr = WIN_GetPtr( hwnd );

  TRACE("(%p)\n", hwnd);

  if (!wndPtr->parent)  /* desktop window */
    {
	  wndPtr->pDriverData = NULL;
	  WIN_ReleasePtr( wndPtr );
	  return TRUE;
    }

  wndPtr->pDriverData = QUARTZDRV_CreateNewWindow(cs->lpszName,
												  cs->x, cs->y, cs->cx, cs->cy);
  WIN_ReleasePtr(wndPtr);

    /* Call the WH_CBT hook */

    hwndLinkAfter = ((cs->style & (WS_CHILD|WS_MAXIMIZE)) == WS_CHILD) ? HWND_BOTTOM : HWND_TOP;

    cbtc.lpcs = cs;
    cbtc.hwndInsertAfter = hwndLinkAfter;
    if (HOOK_CallHooks( WH_CBT, HCBT_CREATEWND, (WPARAM)hwnd, (LPARAM)&cbtc, unicode ))
    {
        TRACE("CBT-hook returned !0\n");
        return FALSE;
    }

    if (unicode)
    {
        ret = SendMessageW( hwnd, WM_NCCREATE, 0, (LPARAM)cs );
        if (ret) ret = (SendMessageW( hwnd, WM_CREATE, 0, (LPARAM)cs ) != -1);
    }
    else
    {
        ret = SendMessageA( hwnd, WM_NCCREATE, 0, (LPARAM)cs );
        if (ret) ret = (SendMessageA( hwnd, WM_CREATE, 0, (LPARAM)cs ) != -1);
    }
    return ret;
}

/***********************************************************************
 *		DestroyWindow   (QUARTZDRV.@)
 */
BOOL QUARTZDRV_DestroyWindow( HWND hwnd )
{
  extern void ReleaseWindow(void *); /* wrong prototype */

  WND *wndPtr = WIN_GetPtr(hwnd);
  if (wndPtr->pDriverData)
	ReleaseWindow(wndPtr->pDriverData);
  WIN_ReleasePtr(wndPtr);

  return TRUE;
}

/***********************************************************************
 *		GetDC   (QUARTZDRV.@)
 *
 * Set the drawable, origin and dimensions for the DC associated to
 * a given window.
 */
BOOL QUARTZDRV_GetDC( HWND hwnd, HDC hdc, HRGN hrgn, DWORD flags )
{
  FIXME(": stub\n");
  return TRUE;
}


/***********************************************************************
 *		SetWindowPos   (QUARTZDRV.@)
 */
BOOL QUARTZDRV_SetWindowPos( WINDOWPOS *winpos )
{
  WND *wndPtr;
  RECT newWindowRect;
  if (!(wndPtr = WIN_FindWndPtr(winpos->hwnd)) || !wndPtr->pDriverData)
	return FALSE;

  /* Send WM_WINDOWPOSCHANGING message */

  if (!(winpos->flags & SWP_NOSENDCHANGING))
	SendMessageA( wndPtr->hwndSelf, WM_WINDOWPOSCHANGING, 0, (LPARAM)winpos);

  newWindowRect = wndPtr->rectWindow;

  // send WM_NCCALCSIZE

  if (!(winpos->flags & SWP_NOMOVE)) {
	extern void CARBON_MoveWindow(void *, short, short, BOOL);
	CARBON_MoveWindow(wndPtr->pDriverData, winpos->x, winpos->y,
					  !(winpos->flags & SWP_NOACTIVATE));
  }
  if (!(winpos->flags & SWP_NOSIZE)) {
	extern void CARBON_SizeWindow(void *,short, short, BOOL);
	CARBON_SizeWindow(wndPtr->pDriverData, winpos->cx, winpos->cy,
					  !(winpos->flags & SWP_NOREDRAW));
  }

  if (!(winpos->flags & SWP_NOZORDER) && winpos->hwnd != winpos->hwndInsertAfter) {
	extern void CARBON_SendBehind(void *, void*);
	extern void CARBON_SetWindowGroup(void*, void*);
	extern void CARBON_BringToFront(void*);
	switch ((long)winpos->hwndInsertAfter) {
	case (long)HWND_BOTTOM:
	  CARBON_SendBehind(wndPtr->pDriverData, NULL);
	  break;
	case (long)HWND_NOTOPMOST:
	  CARBON_SetWindowGroup(wndPtr->pDriverData, qd_nontopmost_group);
	  break;
	case (long)HWND_TOP:
	  CARBON_BringToFront(wndPtr->pDriverData);
	  break;
	case (long)HWND_TOPMOST:
	  CARBON_SetWindowGroup(wndPtr->pDriverData, qd_topmost_group);
	  break;
	default: 
	  {
		WND *other = WIN_FindWndPtr(winpos->hwndInsertAfter);
		if (other && other->pDriverData) {
		  CARBON_SendBehind(wndPtr->pDriverData, other->pDriverData);
		}
		WIN_ReleaseWndPtr(other);
	  }
	}
  }
  if( winpos->flags & SWP_SHOWWINDOW ) {
	extern void CARBON_ShowWindow(void *);
	WIN_SetStyle( winpos->hwnd, wndPtr->dwStyle | WS_VISIBLE );
	CARBON_ShowWindow(wndPtr->pDriverData);
  } else if( winpos->flags & SWP_HIDEWINDOW ) {
	extern void CARBON_HideWindow(void *);
	WIN_SetStyle( winpos->hwnd, wndPtr->dwStyle & ~WS_VISIBLE );
	CARBON_HideWindow(wndPtr->pDriverData);
  }

  /* And last, send the WM_WINDOWPOSCHANGED message */

  if (!(winpos->flags & SWP_NOSENDCHANGING))
	SendMessageA( winpos->hwnd, WM_WINDOWPOSCHANGED, 0, (LPARAM)winpos );

  WIN_ReleaseWndPtr(wndPtr);
  return TRUE;
}

/***********************************************************************
 *              ShowWindow   (QUARTZDRV.@)
 *
 *Sets the specified windows' show state.
 */
BOOL QUARTZDRV_ShowWindow( HWND hwnd, INT cmd )
{
  extern void CARBON_HideWindow(void *);
  extern void CARBON_ShowWindow(void *);
  extern void CARBON_CollapseWindow(void *, BOOL collapse);
  WND* 	wndPtr = WIN_FindWndPtr( hwnd );
  if (!(wndPtr && wndPtr->pDriverData))
	return FALSE;

  FIXME(": stub\n");
  switch (cmd) {
  case SW_HIDE:
	CARBON_HideWindow(wndPtr->pDriverData);
	break;
  case SW_MINIMIZE:
	CARBON_CollapseWindow(wndPtr->pDriverData, TRUE);
	break;
  case SW_RESTORE:
	CARBON_CollapseWindow(wndPtr->pDriverData, FALSE);
  default:
	CARBON_ShowWindow(wndPtr->pDriverData);
  }

  return TRUE;
}
